' SED_PBWIN - Use the PBWIN compiler
#COMPILE EXE
#DIM ALL
#INCLUDE ONCE "GLFW.INC"
#INCLUDE ONCE "GDIPUTILS.INC"

TYPE VERTEX
  ' 3D Coordinates
  x AS SINGLE
  y AS SINGLE
  z AS SINGLE
  ' Texture Coordinates
  u AS SINGLE
  v AS SINGLE
END TYPE

' =======================================================================================
' Main
' =======================================================================================
FUNCTION PBMAIN () AS LONG

   LOCAL hr, nWidth, nHeight, running, frames, x, y AS LONG
   LOCAL t, t0, fps AS DOUBLE
   LOCAL szTitlestr AS ASCIIZ * 200
   LOCAL strTextureData AS STRING
   LOCAL TextureWidth, TextureHeight AS LONG
   DIM   glHandles(2) AS DWORD

   LOCAL x_m, y_m, z_m, u_m, v_m AS SINGLE
   LOCAL z, xtrans, ztrans, ytrans AS SINGLE
   LOCAL lookupdown, sceneroty AS SINGLE
   LOCAL loop_m, filter, numtriangles AS LONG
   LOCAL xpos, zpos, yrot AS SINGLE
   LOCAL heading, walkbias, walkbiasangle AS SINGLE
   LOCAL blend, light, bp, fp, lp AS LONG

   ' Initialize GLFW
   glfwInit

   ' Open OpenGL window
   IF ISFALSE glfwOpenWindow(640, 480, 0, 0, 0, 0, 16, 0, %GLFW_WINDOW) THEN
      glfwTerminate
      EXIT FUNCTION
   END IF

   ' Load bitmap texture from disk
   hr = GdiPlusLoadTexture("mud.bmp", TextureWidth, TextureHeight, strTextureData, %TRUE)

   ' Assign handles
   glGenTextures 3, glHandles(0)

   ' Create Nearest Filtered Texture
   glBindTexture %GL_TEXTURE_2D, glHandles(0)
   glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MAG_FILTER, %GL_NEAREST
   glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MIN_FILTER, %GL_NEAREST
   glTexImage2D %GL_TEXTURE_2D, 0, 3, TextureWidth, TextureHeight, 0, _
                %GL_RGBA, %GL_UNSIGNED_BYTE, BYVAL STRPTR(strTextureData)

   ' Create Linear Filtered Texture
   glBindTexture %GL_TEXTURE_2D, glHandles(1)
   glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MAG_FILTER, %GL_LINEAR
   glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MIN_FILTER, %GL_LINEAR
   glTexImage2D %GL_TEXTURE_2D, 0, 3, TextureWidth, TextureHeight, 0, _
                %GL_RGBA, %GL_UNSIGNED_BYTE, BYVAL STRPTR(strTextureData)

   ' Create MipMapped Texture
   glBindTexture %GL_TEXTURE_2D, glHandles(2)
   glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MAG_FILTER, %GL_LINEAR
   glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MIN_FILTER, %GL_LINEAR_MIPMAP_NEAREST
   gluBuild2DMipmaps %GL_TEXTURE_2D, 3, TextureWidth, TextureHeight, _
                     %GL_RGBA, %GL_UNSIGNED_BYTE, BYVAL STRPTR(strTextureData)

   ' Enable texture mapping
   glEnable(%GL_TEXTURE_2D)
   ' Select smooth shading
   glShadeModel %GL_SMOOTH
   ' Specify clear values for the color buffers
   glClearColor 0.0!, 0.0!, 0.0!, 0.0!
   ' Specify the clear value for the depth buffer
   glClearDepth 1.0!
   ' The type of depth test to do
   glDepthFunc %GL_LESS
   ' Enables depth testing
   glEnable %GL_DEPTH_TEST
	' Really nice perspective calculations
   glHint %GL_PERSPECTIVE_CORRECTION_HINT, %GL_NICEST

   ' Set the blending function for translucency
   glBlendFunc %GL_SRC_ALPHA,%GL_ONE

   ' Setup world
   LOCAL i, c, boom, vert AS LONG
   LOCAL oneline AS STRING

   OPEN "world.txt" FOR INPUT AS #1

   DO UNTIL EOF(1)
      LINE INPUT #1, oneline
      IF LEFT$(oneline, 11) = "NUMPOLLIES " THEN
         numtriangles = VAL(MID$(oneline, 12))
         EXIT DO
      END IF
   LOOP

   REDIM mytriangles(numtriangles * 3 - 1) AS VERTEX

   FOR c = 0 TO numtriangles - 1
      FOR vert = 0 TO 2
         boom = 0
         DO
            LINE INPUT #1, oneline
            oneline = TRIM$(oneline)
            IF oneline <> "" THEN
               IF LEFT$(oneline, 2) <> "//" THEN
                  mytriangles(c * 3 + vert).x = VAL(oneline)
                  i = INSTR(oneline, " ")
                  oneline = TRIM$(MID$(oneline, i))
                  mytriangles(c * 3 + vert).y = VAL(oneline)
                  i = INSTR(oneline, " ")
                  oneline = TRIM$(MID$(oneline, i))
                  mytriangles(c * 3 + vert).z = VAL(oneline)
                  i = INSTR(oneline, " ")
                  oneline = TRIM$(MID$(oneline, i))
                  mytriangles(c * 3 + vert).u = VAL(oneline)
                  i = INSTR(oneline, " ")
                  oneline = TRIM$(MID$(oneline, i))
                  mytriangles(c * 3 + vert).v = VAL(oneline)
                  boom = 1
               END IF
            END IF
         LOOP UNTIL boom = 1
      NEXT
   NEXT
   CLOSE #1

   REDIM LightAmbient(3) AS SINGLE
   REDIM LightDiffuse(3) AS SINGLE
   REDIM LightPosition(3) AS SINGLE

   LightAmbient(0)  = 0.5! : LightAmbient(1)  = 0.5! : LightAmbient(2)  = 0.5! : LightAmbient(3)  = 1.0! ' Ambient Light
   LightDiffuse(0)  = 1.0! : LightDiffuse(1)  = 1.0! : LightDiffuse(2)  = 1.0! : LightDiffuse(3)  = 1.0! ' Diffuse Light
   LightPosition(0) = 0.0! : LightPosition(1) = 0.0! : LightPosition(2) = 2.0! : LightPosition(3) = 1.0! ' Light Position

   glLightfv %GL_LIGHT0, %GL_AMBIENT, LightAmbient(0)
   glLightfv %GL_LIGHT0, %GL_DIFFUSE, LightDiffuse(0)
   glLightfv %GL_LIGHT0, %GL_POSITION, LightPosition(0)
   glEnable %GL_LIGHT0

   ' Enable sticky keys
   glfwEnable %GLFW_STICKY_KEYS

   ' Disable vertical sync (on cards that support it)
   glfwSwapInterval 0

   ' Main loop
   running = %TRUE
   frames = 0
   t0 = glfwGetTime

   DO WHILE running

      ' Get time and mouse position
      t = glfwGetTime
      glfwGetMousePos x, y
      ' Calculate and display FPS (frames per second)
      IF t - t0 > 1.0! OR frames = 0 THEN
         fps = frames / (t-t0)
         wsprintf szTitlestr, "NEHE Lesson 10 (%i FPS)", BYVAL fps
         glfwSetWindowTitle szTitlestr
         t0 = t
         frames = 0
      END IF
      frames = frames + 1

      '  Get window size (may be different than the requested size)
      glfwGetWindowSize nWidth, nHeight
      IF nHeight <= 0 THEN nHeight = 1

      ' Set viewport
      glViewport 0, 0, nWidth, nHeight

      ' Clear the screen and the depth buffers
      glClear %GL_COLOR_BUFFER_BIT OR %GL_DEPTH_BUFFER_BIT

      ' Select and setup the projection matrix
      glMatrixMode %GL_PROJECTION
      glLoadIdentity

      ' Calculate the aspect ratio of the window
      gluPerspective 45.0!, nWidth/nHeight, 0.1!, 100.0!

      ' Select the modelview matrix
      glMatrixMode %GL_MODELVIEW
      glLoadIdentity

      xtrans = -xpos
      ztrans = -zpos
      ytrans = -walkbias - 0.25!

      sceneroty = 360.0! - yrot

      glRotatef lookupdown, 1.0!, 0,0!
      glRotatef sceneroty, 0.0!, 1.0!, 0.0!

      glTranslatef xtrans, ytrans, ztrans
      glBindTexture %GL_TEXTURE_2D, glHandles(filter)

      ' Process each triangle
      FOR loop_m = 0 TO numtriangles - 1 STEP 3
         glBegin %GL_TRIANGLES
         glNormal3f 0.0!, 0.0!, 1.0!
         x_m = mytriangles(loop_m + 0).x
         y_m = mytriangles(loop_m + 0).y
         z_m = mytriangles(loop_m + 0).z
         u_m = mytriangles(loop_m + 0).u
         v_m = mytriangles(loop_m + 0).v
         glTexCoord2f u_m,v_m
         glVertex3f x_m,y_m,z_m

         x_m = mytriangles(loop_m + 1).x
         y_m = mytriangles(loop_m + 1).y
         z_m = mytriangles(loop_m + 1).z
         u_m = mytriangles(loop_m + 1).u
         v_m = mytriangles(loop_m + 1).v
         glTexCoord2f u_m,v_m
         glVertex3f x_m,y_m,z_m

         x_m = mytriangles(loop_m + 2).x
         y_m = mytriangles(loop_m + 2).y
         z_m = mytriangles(loop_m + 2).z
         u_m = mytriangles(loop_m + 2).u
         v_m = mytriangles(loop_m + 2).v
         glTexCoord2f u_m,v_m
         glVertex3f x_m,y_m,z_m
         glEnd
      NEXT

      ' Dinky's light fix ;)
      glLightfv %GL_LIGHT0, %GL_POSITION, LightPosition(0)

      ' Swap buffers
      glfwSwapBuffers

      ' B key
      IF glfwGetKey(66) = %GLFW_PRESS AND ISFALSE bp THEN
         bp = %TRUE
         blend = NOT blend
         IF blend THEN
            glEnable %GL_BLEND
            glDisable %GL_DEPTH_TEST
         ELSE
            glDisable %GL_BLEND
            glEnable %GL_DEPTH_TEST
         END IF
      END IF
      IF glfwGetKey(66) <> %GLFW_PRESS THEN bp = %FALSE

      ' F key
      IF glfwGetKey(70) = %GLFW_PRESS AND ISFALSE fp THEN
         fp = %TRUE
         filter = filter + 1
         IF filter > 2 THEN filter = 0
      END IF
      IF glfwGetKey(70) <> %GLFW_PRESS THEN fp = %FALSE

      ' L key
      IF glfwGetKey(76) = %GLFW_PRESS AND ISFALSE lp THEN
         lp = %TRUE
         light = NOT light
         IF ISFALSE light THEN
            glDisable %GL_LIGHTING
         ELSE
            glEnable %GL_LIGHTING
         END IF
      END IF
      IF glfwGetKey(76) <> %GLFW_PRESS THEN lp = %FALSE

      IF glfwGetKey(%GLFW_KEY_PAGEUP) = %GLFW_PRESS THEN
         z = z - 0.02!
         lookupdown = lookupdown - 1.0!
      END IF

      IF glfwGetKey(%GLFW_KEY_PAGEDOWN) = %GLFW_PRESS THEN
         z = z + 0.02!
         lookupdown = lookupdown + 1.0!
      END IF

      IF glfwGetKey(%GLFW_KEY_UP) = %GLFW_PRESS THEN
         xpos = xpos - Sin(heading * 0.0174532925!) * 0.05!
         zpos = zpos - Cos(heading * 0.0174532925!) * 0.05!
         If walkbiasangle >= 359.0! Then
            walkbiasangle = 0.0!
         Else
            walkbiasangle = walkbiasangle + 10
         End If
         walkbias = Sin(walkbiasangle * 0.0174532925!) / 20.0!
      END IF

      IF glfwGetKey(%GLFW_KEY_DOWN) = %GLFW_PRESS THEN
         xpos = xpos + Sin(heading * 0.0174532925!) * 0.05!
         zpos = zpos + Cos(heading * 0.0174532925!) * 0.05!
         If walkbiasangle <= 1.0! Then
           walkbiasangle = 359.0!
         Else
          walkbiasangle = walkbiasangle - 10
         End If
         walkbias = Sin(walkbiasangle * 0.0174532925!) / 20.0!
      END IF

      IF glfwGetKey(%GLFW_KEY_RIGHT) = %GLFW_PRESS THEN
         heading = heading - 1.0!
         yrot = heading
      END IF

      IF glfwGetKey(%GLFW_KEY_LEFT) = %GLFW_PRESS THEN
         heading = heading + 1.0!
         yrot = heading
      END IF

      ' Check if the ESC key was pressed or the window was closed
      running = NOT glfwGetKey(%GLFW_KEY_ESC) AND glfwGetWindowParam(%GLFW_OPENED)

   LOOP

   ' Delete the texture
   glDeleteTextures(3, glHandles(0))

   ' Close OpenGL window and terminate GLFW
   glfwTerminate

END FUNCTION
' =======================================================================================
